@@ -2,28 +2,23 @@
 # Make sure it was checked out with git clone --recursive ...
 # If not, update the submodules
 
-for sub in be13_api dfxml 
+mkdir -p build-aux
+
+for sub in be20_api be20_api/dfxml_cpp
 do
   if [ ! -r src/$sub/.git ] ;  then
-    echo bringing in submodules
-    echo next time check out with git clone --recursive
-    git submodule init
-    git submodule update
+    echo submodule $sub is not present.
+    exit 1
   fi
 done
 
 # have automake do an initial population if necessary
-if [ ! -e config.guess -o ! -e config.sub -o ! -e install-sh -o ! -e missing -o ! -e test-driver ]; then
-    autoheader -f
-    touch NEWS README AUTHORS ChangeLog
-    touch stamp-h
-    aclocal -I m4
-    autoconf -f
-    #libtoolize || glibtoolize
-    automake --add-missing --copy
-else
-    autoreconf -f
-fi
+autoheader -f
+touch NEWS README AUTHORS ChangeLog
+touch stamp-h
+aclocal -I m4
+autoconf -f
+automake --add-missing --copy
 
 # We were very excited about AddressSanitizer.
 # This is how to enable it...
