@@ -18,12 +18,12 @@
 #include <algorithm>
 #include "config.h"
 
-#include "utf8.h" // for reading UTF16 and UTF32
-#include "unicode_escape.h" // for validating or escaping UTF8
+#include "be20_api/utf8.h" // for reading UTF16 and UTF32
+#include "be20_api/utils.h"
+#include "be20_api/unicode_escape.h" // for validating or escaping UTF8
+#include "exif_entry.h"
 #include "exif_reader.h"
 
-// #define DEBUG 1
-
 static const uint32_t OPT_MAX_IFD_ENTRIES = 1000;	// don't parse more than this
 static const uint32_t OPT_MAX_EXIF_BYTES = 64 * 1024; // rough sanity limit
 static const bool opt_exif_suppress_unknown_entry_types = true; // suppress undefined entry types
@@ -70,10 +70,20 @@ static std::string get_exif_slong(tiff_handle_t &tiff_handle, uint32_t ifd_entry
 static std::string get_exif_srational(tiff_handle_t &tiff_handle, uint32_t ifd_entry_offset);
 
 exif_entry::exif_entry(uint16_t ifd_type_, const std::string &name_, const std::string &value_)
-    :ifd_type(ifd_type_), name(name_), value(value_) {
+    :ifd_type(ifd_type_), name(name_), value(value_)
+{
 }
 
-exif_entry::exif_entry(const exif_entry &that):ifd_type(that.ifd_type), name(that.name), value(that.value) {
+// copy
+exif_entry::exif_entry(const exif_entry &that):
+    ifd_type(that.ifd_type), name(that.name), value(that.value)
+{
+}
+
+// move and copy are the same
+exif_entry::exif_entry(const exif_entry &&that):
+    ifd_type(that.ifd_type), name(that.name), value(that.value)
+{
 }
 
 exif_entry::~exif_entry() {
@@ -101,20 +111,19 @@ const std::string exif_entry::get_full_name() const {
     case IFD1_INTEROPERABILITY:
         return "ifd1.interoperability." + name;
     default:
-        std::cerr << "Program state errror: Invalid ifd type " << ifd_type << "\n";
-        assert(0);
+        return "unknown." + name;
     }
-    return "ERROR";			// required to avoid compiler warning
 }
- 
-/** 
+
+/**
  * parse_ifd_entries() extracts entries from an offset given its type.
  * Throws exif_failure_exception if the exif data state is determined to be invalid
  * such that further entry parsing would be invalid.
  * @Throws exif_failure_exception_t
  */
 void entry_reader::parse_ifd_entries(ifd_type_t ifd_type, tiff_handle_t &tiff_handle,
-				     size_t ifd_offset, entry_list_t &entries) {
+				     size_t ifd_offset, entry_list_t &entries)
+{
     uint16_t num_entries = tiff_ifd_reader::get_num_entries(tiff_handle, ifd_offset);
 #ifdef DEBUG
     std::cout << "exif_entry.parse_ifd_entries ifd type " << (uint32_t)ifd_type << ", num entries: " << num_entries << " from offset " << ifd_offset << "\n";
@@ -145,7 +154,7 @@ void entry_reader::parse_ifd_entries(ifd_type_t ifd_type, tiff_handle_t &tiff_ha
 
 void entry_reader::parse_entry(ifd_type_t ifd_type, tiff_handle_t &tiff_handle,
 			       size_t ifd_entry_offset, entry_list_t &entries) {
-/** 
+/**
  * parse_ifd_entry() parses one entry and places it into entries.
  * Throws exif_failure_exception if the exif data state is determined to be invalid
  * such that further entry parsing would be invalid.
@@ -1005,7 +1014,7 @@ static std::string get_generic_name(tiff_handle_t &tiff_handle, uint32_t ifd_ent
     uint16_t entry_tag;
     try {
         entry_tag = tiff_handle.sbuf->get16u(ifd_entry_offset + 0, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         entry_tag = 0;
     }
     std::stringstream ss;
@@ -1028,13 +1037,13 @@ static void add_entry(ifd_type_t ifd_type, const std::string &name, const std::s
     std::cout << "exif_entry.add_entry value: '" << value << "'\n";
 #endif
 
-    entries.push_back(new exif_entry(ifd_type, name, value));
+    entries.push_back(std::make_unique<exif_entry>(ifd_type, name, value));
 }
 
 inline static uint16_t get_entry_tag(tiff_handle_t &tiff_handle, uint32_t ifd_entry_offset) {
     try {
         return tiff_handle.sbuf->get16u(ifd_entry_offset + 0, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         return 0;
     }
 }
@@ -1042,7 +1051,7 @@ inline static uint16_t get_entry_tag(tiff_handle_t &tiff_handle, uint32_t ifd_en
 inline static uint16_t get_entry_type(tiff_handle_t &tiff_handle, uint32_t ifd_entry_offset) {
     try {
         return tiff_handle.sbuf->get16u(ifd_entry_offset + 2, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         return 0;
     }
 }
@@ -1053,7 +1062,7 @@ inline static uint32_t get_entry_count(tiff_handle_t &tiff_handle, uint32_t ifd_
     uint32_t requested_count;
     try {
         requested_count = tiff_handle.sbuf->get32u(ifd_entry_offset + 4, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         requested_count = 0;
     }
     return requested_count;
@@ -1064,7 +1073,7 @@ inline static uint16_t get_data_offset(tiff_handle_t &tiff_handle, uint32_t ifd_
     uint16_t entry_type;
     try {
         entry_type = tiff_handle.sbuf->get16u(ifd_entry_offset + 2, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         return 0;
     }
 
@@ -1092,7 +1101,7 @@ inline static uint16_t get_data_offset(tiff_handle_t &tiff_handle, uint32_t ifd_
     uint32_t requested_count;
     try {
         requested_count = tiff_handle.sbuf->get32u(ifd_entry_offset + 4, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         requested_count = 0;
     }
 
@@ -1107,7 +1116,7 @@ inline static uint16_t get_data_offset(tiff_handle_t &tiff_handle, uint32_t ifd_
         // look up the value offset
         try {
             return tiff_handle.sbuf->get32u(ifd_entry_offset + 8, tiff_handle.byte_order);
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // return an out-of-range value
             return ifd_entry_offset + 12;
         }
@@ -1118,7 +1127,7 @@ inline static uint16_t get_data_offset(tiff_handle_t &tiff_handle, uint32_t ifd_
 inline static uint32_t get_ifd_offset(tiff_handle_t &tiff_handle, uint32_t ifd_entry_offset) {
     try {
         return tiff_handle.sbuf->get32u(ifd_entry_offset + 8, tiff_handle.byte_order);
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         // return an out-of-range value
         return ifd_entry_offset + 12;
     }
@@ -1181,16 +1190,12 @@ static std::string get_possible_utf8(const sbuf_t &sbuf, size_t count) {
         // this is an uninteresting sequence, so return ""
         return "";
     } else {
-        std::string s;
-        sbuf.getUTF8WithQuoting(0, count, s);
+        std::string s = sbuf.getUTF8(0, count);
+        validateOrEscapeUTF8(s,true,false,true);
         return s;
     }
 }
 
-template <typename u16bit_iterator, typename octet_iterator>
-octet_iterator all_utf16to8 (u16bit_iterator start, u16bit_iterator end, octet_iterator result);
-std::string all_utf16to8 (std::wstring utf16_string);
-
 // read utf16 and return unvalidated utf8
 std::string get_possible_utf16(const sbuf_t sbuf, size_t count, sbuf_t::byte_order_t byte_order) {
 
@@ -1201,15 +1206,11 @@ std::string get_possible_utf16(const sbuf_t sbuf, size_t count, sbuf_t::byte_ord
     }
 
     // get wstring accounting for byte order
-    std::wstring wstr;
-    sbuf.getUTF16(0, count, byte_order, wstr);
+    std::wstring wstr = sbuf.getUTF16(0, count, byte_order);
 
     // convert wstring to string
-    std::string utf8_string = all_utf16to8(wstr);
+    std::string utf8_string = safe_utf16to8(wstr);
 
-#ifdef DEBUG
-    std::cout << "exif_entry.get_possible_utf16 utf8_string (escaped): '" << validateOrEscapeUTF8(utf8_string, true, true) << "'\n";
-#endif
     return utf8_string;
 }
 
@@ -1230,7 +1231,7 @@ std::string get_possible_utf32(const sbuf_t &sbuf, size_t count, sbuf_t::byte_or
             code_point = sbuf.get32u(i * 4, byte_order);
             try {
                 result = utf8::append(code_point, result);
-            } catch (utf8::invalid_code_point) {
+            } catch (const utf8::invalid_code_point &) {
 
                 // invalid code point so put in the byte values directly,
                 // disregarding endian convention
@@ -1239,7 +1240,7 @@ std::string get_possible_utf32(const sbuf_t &sbuf, size_t count, sbuf_t::byte_or
                 utf8_string += (uint8_t)code_point/0x10000;
                 utf8_string += (uint8_t)code_point/0x1000000;
             }
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // at end of buffer
             break;
         }
@@ -1247,48 +1248,6 @@ std::string get_possible_utf32(const sbuf_t &sbuf, size_t count, sbuf_t::byte_or
     return utf8_string;
 }
 
-// convert all possible utf16, along with any illegal values
-std::string all_utf16to8 (std::wstring utf16_string) {
-    std::string utf8_string;
-    std::back_insert_iterator<std::basic_string<char> > result = back_inserter(utf8_string);
-
-    std::wstring::const_iterator start = utf16_string.begin();
-    std::wstring::const_iterator end = utf16_string.end();
-
-//template <typename u16bit_iterator, typename octet_iterator>
-//octet_iterator all_utf16to8 (u16bit_iterator start, u16bit_iterator end, octet_iterator result)
-
-    while (start != end) {
-        uint32_t code_point = utf8::internal::mask16(*start++);
-        // Take care of surrogate pairs first
-        if (utf8::internal::is_lead_surrogate(code_point)) {
-            if (start != end) {
-                uint32_t trail_surrogate = utf8::internal::mask16(*start++);
-                if (utf8::internal::is_trail_surrogate(trail_surrogate)) {
-                    code_point = (code_point << 10) + trail_surrogate + utf8::internal::SURROGATE_OFFSET;
-                } else {
-                    // there was an error, but keep invalid code_point anyway rather than raise an exception
-                }
-            } else {
-                // there was an error, but keep invalid code_point anyway rather than raise an exception
-            }
-
-        }
-        // Lone trail surrogate
-        else if (utf8::internal::is_trail_surrogate(code_point)) {
-            // there was an error, but keep invalid code_point anyway rather than raise an exception
-        }
-
-        try {
-            result = utf8::append(code_point, result);
-        } catch (utf8::invalid_code_point) {
-            // invalid code point so put in unescaped byte values, disregarding endian convention
-            utf8_string += (uint8_t)code_point;
-            utf8_string += (uint8_t)code_point/0x100;
-        }
-    }
-    return utf8_string;
-}
 
 // ************************************************************
 // END This section provides interfaces get_possible_utf16, and get_possible_utf32.
@@ -1303,7 +1262,7 @@ std::string all_utf16to8 (std::wstring utf16_string) {
  * @Throws exif_failure_exception_t
  */
 static std::string get_exif_byte(tiff_handle_t &tiff_handle, uint32_t ifd_entry_offset) {
- 
+
     uint32_t count = get_entry_count(tiff_handle, ifd_entry_offset);
     uint32_t offset = get_data_offset(tiff_handle, ifd_entry_offset);
 
@@ -1316,14 +1275,16 @@ static std::string get_exif_byte(tiff_handle_t &tiff_handle, uint32_t ifd_entry_
         std::stringstream ss;
         try {
 	    ss << (int)tiff_handle.sbuf->get8u(offset);
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // add nothing to ss
         }
         return ss.str();
 
     } else {
         // count is not 1 so return the bytes as utf8
-        return get_possible_utf8(*(tiff_handle.sbuf)+offset, count);
+        // TODO: replace this with sbuf mechanism to get UTF8?
+        // return tiff_handle.getUTF8(offset, count, utf8_string)
+        return get_possible_utf8(tiff_handle.sbuf->slice(offset), count);
     }
 }
 
@@ -1350,12 +1311,12 @@ static std::string get_exif_ascii(tiff_handle_t &tiff_handle, uint32_t ifd_entry
         if (count > 0 && tiff_handle.sbuf->get8u(offset + count - 1) == 0) {
 	    count--;
         }
-    } catch (sbuf_t::range_exception_t &e) {
+    } catch (const sbuf_t::range_exception_t &e) {
         // at end so there is nothing to strip
     }
 
     // although 7-bit ascii is expected, parse using UTF-8
-    return get_possible_utf8(*(tiff_handle.sbuf)+offset, count);
+    return get_possible_utf8(tiff_handle.sbuf->slice(offset), count);
 }
 
 // EXIF_SHORT uint16
@@ -1378,16 +1339,16 @@ static std::string get_exif_short(tiff_handle_t &tiff_handle, uint32_t ifd_entry
         std::stringstream ss;
         try {
 	    ss << (int)tiff_handle.sbuf->get16u(offset, tiff_handle.byte_order);
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // add nothing to ss
         }
         return ss.str();
 
     } else {
         // count is not 1 so print the uint16_t bytes as utf8
-        std::string s = get_possible_utf16(*(tiff_handle.sbuf)+offset, count, tiff_handle.byte_order);
+        std::string s = get_possible_utf16( tiff_handle.sbuf->slice(offset), count, tiff_handle.byte_order);
 #ifdef DEBUG
-        std::cout << "exif_entry.get_exif_short (escaped): '" << validateOrEscapeUTF8(s, true, true) << "'\n";
+        std::cout << "exif_entry.get_exif_short (escaped): '" << validateOrEscapeUTF8(s, true, false) << "'\n";
 #endif
         return s;
     }
@@ -1414,16 +1375,16 @@ static std::string get_exif_long(tiff_handle_t &tiff_handle, uint32_t ifd_entry_
         std::stringstream ss;
         try {
 	    ss << (uint32_t)tiff_handle.sbuf->get32u(offset, tiff_handle.byte_order);
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // add nothing to ss
         }
         return ss.str();
 
     } else {
         // count is not 1 so print the uint32_t bytes as utf8
-        std::string s = get_possible_utf32(*(tiff_handle.sbuf)+offset, count, tiff_handle.byte_order);
+        std::string s = get_possible_utf32( tiff_handle.sbuf->slice(offset), count, tiff_handle.byte_order);
 #ifdef DEBUG
-        std::cout << "exif_entry.get_exif_short (escaped): '" << validateOrEscapeUTF8(s, true, true) << "'\n";
+        std::cout << "exif_entry.get_exif_short (escaped): '" << validateOrEscapeUTF8(s, true, false) << "'\n";
 #endif
         return s;
     }
@@ -1453,7 +1414,7 @@ static std::string get_exif_rational(tiff_handle_t &tiff_handle, uint32_t ifd_en
 	    if (i + 1 < count) {
 	        ss << " ";
 	    }
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             break;
         }
     }
@@ -1485,7 +1446,7 @@ static  std::string get_exif_slong(tiff_handle_t &tiff_handle, uint32_t ifd_entr
     for (uint32_t i=0; i<count; i++) {
         try {
 	    ss << tiff_handle.sbuf->get32i(offset + i * 4, tiff_handle.byte_order);
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             // at end
             break;
         }
@@ -1521,10 +1482,9 @@ static std::string get_exif_srational(tiff_handle_t &tiff_handle, uint32_t ifd_e
 	    if (i + 1 < count) {
 	        ss << " ";
 	    }
-        } catch (sbuf_t::range_exception_t &e) {
+        } catch (const sbuf_t::range_exception_t &e) {
             break;
         }
     }
     return ss.str();
 }
-
