@@ -1,46 +1,65 @@
 #!/usr/bin/env python3
 # coding=UTF-8
 """
-Regression system: 
+Original BE1.5.3 regression system:
 
-A basic framework for running bulk_extractor and viewing the results.
+A basic framework for running bulk_extractor and comparing the number of results for each kind of feature file
+to hard-coded answers.
 
 Options include:
 
+ --download - download files needed
  --verify  - verify a single BE report
  --tune    - runs bulk_extractor under a variety of conditions
  - Total number of features are reported and compared with the archives.
 """
 
-__version__ = "1.5.0"
+__version__ = "1.5.3"
 
-b'This module needs Python 2.7 or later.'
 
-import os,sys
-try:
-    sys.path.append(os.getenv("DOMEX_HOME") + "/src/bulk_extractor/trunk/python/") # add the library
-except TypeError as e:
-    pass
-sys.path.append("../python/")      # add the library
-sys.path.append("python/")      # add the library
+import codecs
+import glob
+import logging
+import os
+import os.path
+import subprocess
+import sys
+import time
+import zipfile
 
 from subprocess import Popen,call,PIPE
-import os.path,glob,zipfile,codecs
+
+sys.path.append("../python/")   # add the library
+sys.path.append("python/")      # add the library
 import bulk_extractor_reader
 
-corp_default = "/corp"
 
+CORP_ENV     = "DOMEX_CORP"     # default environment variable where corpus is located
+CORP_DEFAULT = "/corp"
+
+DOWNLOAD_URL = "http://downloads.digitalcorpora.org/corpora/drives"
+
+IMAGE_UBNIST1 = "ubnist1"
+IMAGE_EMAILS  = "emails"
+IMAGE_DOMEXUSERS = "domexusers"
+
+IMAGE_PATH = {IMAGE_UBNIST1:"nps-2009-ubnist1/ubnist1.gen3.raw",
+              IMAGE_EMAILS:"nps-2010-emails/nps-2010-emails.E01",
+              IMAGE_DOMEXUSERS:"nps-2009-domexusers/nps-2009-domexusers.E01"}
+
+DEFAULT_INFILE = IMAGE_UBNIST1
+FAST_INFILE    = IMAGE_EMAILS
+FULL_INFILE    = IMAGE_DOMEXUSERS
+exe            = "src/bulk_extractor"
 
-default_infile   = "nps-2009-ubnist1/ubnist1.gen3.raw"
-fast_infile      = "nps-2010-emails/nps-2010-emails.raw"
-full_infile      = "nps-2009-domexusers/nps-2009-domexusers.raw"
-exe              = "src/bulk_extractor"
 if not os.path.exists(exe):
     exe = "../src/bulk_extractor"
 
-nps_drives_path = "/nps/drives/"
 BOM = codecs.BOM_UTF8.decode('utf-8')
 
+def image_path(name):
+    return os.path.join(os.environ.get(CORP_ENV,CORP_DEFAULT), IMAGE_PATH[name])
+
 # Performance tuning
 
 MiB = 1024*1024
@@ -57,7 +76,7 @@ tune_marginsize_end   = 4 * MiB
 tune_marginsize_step  = 1 * MiB
 
 
-answers = {"ubnist1.gen3":{"ALERTS_found.txt":88,
+be153_counts = {"ubnist1.gen3":{"ALERTS_found.txt":88,
                            "bulk_tags.txt":7477796,
                            "ccn.txt":1,
                            "domain.txt":233055,
@@ -94,7 +113,7 @@ def get_scanners(exe):
         if len(atoms)>0 and atoms[0] in ['-x','-e']:
             scanners.add(atoms[1])
     return scanners
-    
+
 
 def ptime(t):
     r = ""
@@ -109,17 +128,6 @@ def ptime(t):
     r += "%d sec " % t
     return r
 
-def find_file(fn):
-    if os.path.exists(fn): return fn
-    chk = []
-    for ne in ('.E01','.aff','.000','.001'):
-        tfn = fn.replace(".raw",ne)
-        if os.path.exists(tfn):
-            return tfn
-        chk.append([tfn])
-    raise RuntimeError("Cannot find file "+tfn)
-
-
 if sys.version_info < (2,7):
     raise "Requires Python 2.7 or above"
 
@@ -198,11 +206,11 @@ def reproduce_flags(outdir):
     return "-Y {offset} {filename}".format(offset=offset, filename=filename)
 
 def analyze_warning(fnpart,fn,lines):
-    if fnpart not in answers:
+    if fnpart not in be153_counts:
         return "(No answers for {})".format(fnpart)
-    if fn not in answers[fnpart]:
+    if fn not in be153_counts[fnpart]:
         return "(No answer for {})".format(fn)
-    ref = answers[fnpart][fn]
+    ref = be153_counts[fnpart][fn]
     if ref==lines: return "OK"
     if lines<ref: return "LOW (expected {})".format(ref)
     return "HIGH (expected {})".format(ref)
@@ -226,10 +234,10 @@ def analyze_reportxml(xmldoc):
             print("*** error: pos0={} was ended by threadid {} but ended by threadid {}".format(pos0,proc[pos0],threadid))
         else:
             del proc[pos0]
-    
+
     for (pos0,threadid) in proc.items():
         print("*** error: pos0={} was started by threadid {} but never ended".format(pos0,threadid))
-    
+
     scanner_times = []
     scanners = xmldoc.getElementsByTagName("scanner_times")[0]
     total = 0
@@ -246,8 +254,8 @@ def analyze_reportxml(xmldoc):
     for (name,calls,seconds) in scanner_times:
         print("  {:>25}  {:8.0f}  {:12.4f}  {:12.4f}  {:5.2f}%".format(
                 name,calls,seconds,seconds/calls,100.0*seconds/total))
-    
-    
+
+
 def analyze_outdir(outdir):
     """Print statistics about an output directory"""
     print("Analyze {}".format(outdir))
@@ -255,10 +263,10 @@ def analyze_outdir(outdir):
     b = bulk_extractor_reader.BulkReport(outdir)
     print("bulk_extractor version: {}".format(b.version()))
     print("Image filename:         {}".format(b.image_filename()))
-    
+
     # Print which scanners were run and how long they took
     analyze_reportxml(b.xmldoc)
-    
+
     hfns = list(b.histogram_files()) # histogram files
     print("")
     print("Histogram Files:        {}".format(len(hfns)))
@@ -279,7 +287,7 @@ def analyze_outdir(outdir):
 
     fnpart = ".".join(b.image_filename().split('/')[-1].split('.')[:-1])
 
-    ffns = sorted(list(b.feature_files())) 
+    ffns = sorted(list(b.feature_files()))
     if ffns:
         features = {}
         print("")
@@ -291,7 +299,7 @@ def analyze_outdir(outdir):
                     lines += 1
                     features[fn] = lines
             print("  {:>25} features: {:>12,}  {}".format(fn,lines,analyze_warning(fnpart,fn,lines)))
-                    
+
         # If there is a SQLite database, analyze that too!
     if args.featurefile and args.featuresql:
         import sqlite3
@@ -325,7 +333,7 @@ def analyze_outdir(outdir):
                                 #pass
                                 print("feature {} {} not in table {} ({})".format(path,feature,table,ct))
                             count += 1
-                            if count>args.featuretest: 
+                            if count>args.featuretest:
                                 break
 
                 except sqlite3.OperationalError as e:
@@ -340,7 +348,7 @@ def make_zip(dname):
     for fname in b.all_files:
         print("  adding {} ...".format(fname))
         z.write(os.path.join(dname,fname),arcname=os.path.basename(fname))
-    
+
 
 def make_outdir(outdir_base):
     counter = 1
@@ -373,8 +381,8 @@ def run_outdir(gdb=False):
     if args.jobs: cargs += ['-j'+str(args.jobs)]
     if args.pagesize: cargs += ['-G'+str(args.pagesize)]
     if args.marginsize: cargs += ['-g'+str(args.marginsize)]
-    
-    cargs += ['-e','all']    # enable 'all' scanners 
+
+    cargs += ['-e','all']    # enable 'all' scanners
     if args.extra:
         while "  " in args.extra:
             args.extra = args.extra.replace("  "," ")
@@ -382,13 +390,15 @@ def run_outdir(gdb=False):
     if args.debug: cargs += ['-d'+str(args.debug)]
 
     if args.find:
-        cargs += ['-r','tests/alert_list.txt']
-        cargs += ['-w','tests/stop_list.txt']
-        cargs += ['-w','tests/stop_list_context.txt']
+        cargs += ['-r','alert_list.txt']
+        cargs += ['-w','stop_list.txt']
+        cargs += ['-w','stop_list_context.txt']
         cargs += ['-f','[a-z\.0-9]*@gsa.gov']
-        cargs += ['-F','tests/find_list.txt']
+        cargs += ['-F','find_list.txt']
 
     if args.image:
+        if not os.path.exists(args.image):
+            raise FileNotFoundError(args.image)
         cargs += [args.image]
     else:
         cargs += ['-R',args.datadir]
@@ -400,14 +410,14 @@ def run_outdir(gdb=False):
             f.write("run ")
             f.write(" ".join(cargs))
             f.write("\n")
-            
+
         cmd = ['gdb','-e',args.exe,'-x','/tmp/cmds']
     else:
         cmd = [args.exe] + cargs
     run(cmd)
     return outdir
 
-             
+
 def sort_outdir(outdir):
     """Sort the output directory with gnu sort"""
     print("Now sorting files in "+outdir)
@@ -428,7 +438,6 @@ def check(fn,lines):
     if lines!=found_lines:
         print("{:10} expected lines: {} found lines: {}".format(fn,found_lines,lines))
 
-
 def asbinary(s):
     ret = ""
     count = 0
@@ -439,7 +448,7 @@ def asbinary(s):
             count = 0
             ret += " "
     return ret
-        
+
 
 FEATURE_FILE = 1
 MAX_OFFSET_SIZE  = 100
@@ -454,7 +463,7 @@ def invalid_feature_file_line(line,fields):
     if len(fields[1])>MAX_FEATURE_SIZE: return "FEATURE > "+str(MAX_FEATURE_SIZE)
     if len(fields[2])>MAX_CONTEXT_SIZE: return "CONTEXT > "+str(MAX_CONTEXT_SIZE)
     return None
-    
+
 def validate_file(f,kind):
     is_kml = f.name.endswith(".kml")
     linenumber = 0
@@ -498,8 +507,8 @@ def validate_report(fn,do_validate=True):
             validate_file(b.open(fn,'rb'),FEATURE_FILE)
     else:
         validate_file(open(fn,'rb'))
-            
-            
+
+
 def identify_filenames(outdir):
     """Run identify_filenames on output using the installed fiwalk"""
     if_outdir = outdir + "-annotated"
@@ -520,7 +529,7 @@ def diff(dname1,dname2):
     def lines_to_set(fn):
         "Read the lines in a file and return them as a set. Ignore the lines beginning with #"
         return set(filter(lambda line:line[0:1]!='#',open(fn).read().split("\n")))
-    
+
     files1 = set(files_in_dir(dname1))
     files2 = set(files_in_dir(dname2))
     if files1.difference(files2):
@@ -533,12 +542,12 @@ def diff(dname1,dname2):
     for fn in sorted(common):
         fn1 = os.path.join(dname1,fn)
         fn2 = os.path.join(dname2,fn)
-        
+
         if fn=="report.xml":
             continue
         if fn=="wordlist.txt" and not args.diffwordlist:
             continue
-        if fn=="packets.pcap": 
+        if fn=="packets.pcap":
             s1 = os.path.getsize(fn1)
             s2 = os.path.getsize(fn2)
             if s1!=s2:
@@ -568,11 +577,9 @@ def diff(dname1,dname2):
             if count1 or count2:
                 print("\n-------------\n")
 
-
-def run_and_analyze():
-    global args
+def run_and_analyze(args):
     t0 = time.time()
-    outdir = run_outdir(args.gdb)
+    outdir = run_outdir(gdb=args.gdb)
     sort_outdir(outdir)
     validate_report(outdir)
     if args.identify_filenames:
@@ -605,10 +612,27 @@ def datacheck():
     if "outlook" in get_scanners(args.exe):
         args.extra += "-e outlook"
     outdir = run_outdir()
-    datacheckreport(outdir)
-        
-def datacheckreport(outdir):
-    # Open the datadir
+    datacheck_checkreport(outdir)
+
+def download():
+    print("Checking downloads:")
+    for name in IMAGE_PATH.keys():
+        path = image_path(name)
+        if not os.path.exists(path):
+            url = os.path.join(DOWNLOAD_URL, IMAGE_PATH[name])
+            print(f"Downloading {url} to {path}")
+            try:
+                os.makedirs( os.path.dirname(path), exist_ok=True)
+            except OSError as e:
+                logging.error("Cannot create directory %s",os.path.dirname(path))
+                logging.error("HINT: Set envrionment variable %s to be the top-level directory of your corpus archive",CORP_ENV)
+                logging.error("getenv(%s)=%s",CORP_ENV,os.environ.get(CORP_ENV,None))
+                exit(1)
+            subprocess.check_call(['curl','-L','-o',path,url])
+
+
+def datacheck_checkreport(outdir):
+    """Reports on whether the output in outdir matches the datacheck report"""
     print("opening ",outdir)
     b = bulk_extractor_reader.BulkReport(outdir)
     found_features = {}
@@ -620,7 +644,7 @@ def datacheckreport(outdir):
             found_features[pos] = feature
     print("Now reading features from data_check.txt")
     not_found = {}
-    report_mismatches = False 
+    report_mismatches = False
     found_count = 0
     for line in open("data_check.txt","rb"):
         y = bulk_extractor_reader.parse_feature_line(line)
@@ -638,22 +662,25 @@ def datacheckreport(outdir):
         print("{} not found {}".format(pos,not_found[pos]))
     print("Total features found: {}".format(found_count))
     print("Total features not found: {}".format(len(not_found)))
-        
-        
+
+
 if __name__=="__main__":
-    import argparse 
+    import argparse
     global args
-    import sys,time
 
     parser = argparse.ArgumentParser(description="Perform regression testing on bulk_extractor",
                                      formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-    parser.add_argument("--gdb",help="run under gdb",action="store_true")
+    parser.add_argument("--download", help="Download the regression images", action='store_true')
+    parser.add_argument("--corp", help="Specification root of corpus. "
+                        "Defaults to location specified by environment variable {} or to {} if "
+                        "environment variable is not present".format(CORP_ENV,CORP_DEFAULT))
+    parser.add_argument("--gdb",help="run under gdb. Useful for crashes",action="store_true")
     parser.add_argument("--debug",help="debug level",type=int)
     parser.add_argument("--outdir",help="output directory base",default=None)
-    parser.add_argument("--exe",help="Executable to run (default {})".format(exe),default=exe)
-    parser.add_argument("--image",help="image to scan (default {})".format(default_infile),default=default_infile)
-    parser.add_argument("--fast",help="Run with "+fast_infile,action="store_true")
-    parser.add_argument("--full",help="Run with "+full_infile,action="store_true")
+    parser.add_argument("--exe",help="Executable to run. ",default=exe)
+    parser.add_argument("--image", help="image to scan. ", default=image_path(DEFAULT_INFILE))
+    parser.add_argument("--fast",help="Run with "+os.path.basename(IMAGE_PATH[FAST_INFILE]),action="store_true")
+    parser.add_argument("--full",help="Run with "+os.path.basename(IMAGE_PATH[FULL_INFILE]),action="store_true")
     parser.add_argument("--jobs",help="Specifies number of worker threads",type=int)
     parser.add_argument("--pagesize",help="Specifies page size",type=int)
     parser.add_argument("--no-find",dest='find',help="Does not do find test (faster)",action="store_false")
@@ -695,10 +722,15 @@ if __name__=="__main__":
     parser.add_argument("--dry-run",help="Don't actually run the program",action='store_true')
     parser.add_argument("--datadircomp",help="Compare two data dirs")
     parser.add_argument("--datacheck",help="Runs BE on the files in Data/ directory and makes sure that all of the features in data_check.txt are found",action='store_true')
-    parser.add_argument("--datacheckreport",help="Checks the files in in Data/ directory and makes sure that all of the features in data_check.txt are found")
+    parser.add_argument("--datacheck_checkreport",help="Checks the files in in Data/ directory and makes sure that all of the features in data_check.txt are found")
 
     args = parser.parse_args()
-    
+
+    if args.download:
+        download()
+        print("All image downloaded")
+        exit(0)
+
     if args.datadircomp:
         dirs = args.datadircomp.split(",")
         datadircomp(dirs[0],dirs[1])
@@ -765,31 +797,27 @@ if __name__=="__main__":
 
     args.outdir += "-"+bulk_extractor_reader.be_version(args.exe)
 
-    if args.datacheck: datacheck() ; exit(1)
+    if args.datacheck:
+        datacheck() ;
+        exit(0)
 
-    if args.datacheckreport: datacheckreport(args.datacheckreport); exit(1)
-
-
-    corp = os.getenv("DOMEX_CORP")
-    if not corp:
-        corp = corp_default
-    drives =  corp + nps_drives_path
+    if args.datacheck_checkreport:
+        datacheck_checkreport(args.datacheck_checkreport);
+        exit(0)
 
     if args.fast:
-        args.image  = fast_infile
+        args.image = image_path(FAST_INFILE)
         if args.extra:
             args.extra += ' '
         else:
             args.extra = ''
         args.extra  += '-G 65536'
+
     if args.full:
-        args.image  = full_infile
+        args.image  = image_path(FULL_INFILE)
 
     args.outdir += "-" + os.path.basename(args.image)
 
-    if not os.path.exists(args.image):
-        args.image = find_file(drives+args.image)
-
     if args.memdebug:
         fn = "/usr/lib/libgmalloc.dylib"
         if os.path.exist(fn):
@@ -814,13 +842,13 @@ if __name__=="__main__":
         os.putenv("MallocCheckHeapAbort","1")
         os.putenv("MallocErrorAbort","1")
         os.putenv("MallocCorruptionAbort","1")
-        run_and_analyze()
+        run_and_analyze(args)
         exit(0)
 
     if args.gprof:
         call(['make','clean'])
         call(['make','CFLAGS=-pg','CXXFLAGS=-pg','LDFLAGS=-pg'])
-        outdir = run_and_analyze()
+        outdir = run_and_analyze(args)
         call(['gprof',program,"gmon.out"],stdout=open(outdir+"/GPROF.txt","w"))
 
     if args.diff:
@@ -863,8 +891,7 @@ if __name__=="__main__":
 
         for j in range(tune_jobs_start,tune_jobs_end+1,tune_jobs_step):
             run_with_parms(j,None,None)
-            
-        exit(0)
-        
-    run_and_analyze()
 
+        exit(0)
+
+    run_and_analyze(args)
